@@ -92,7 +92,6 @@ const char *tgetstr ();
 char *getenv ();
 FILE *readpipe PARAMS ((const char *, ...));
 FILE *writepipe PARAMS ((const char *, ...));
-char *tmpnam ();
 
 /* Declarations.  */
 
@@ -117,7 +116,7 @@ struct option const longopts[] =
   {"start-insert", 1, NULL, 'y'},
   {"statistics"  , 0, NULL, 's'},
   {"terminal"    , 0, NULL, 't'},
-  {"version"     , 0, NULL, 'V'},
+  {"version"     , 0, NULL, 'v'},
   {NULL          , 0, NULL, 0}
 };
 
@@ -157,11 +156,14 @@ copy_mode;
 jmp_buf signal_label;		/* where to jump when signal received */
 int interrupted;		/* set when some signal has been received */
 
-/* Guarantee some value for L_tmpnam.  */
+/* Guarantee some value for L_tmpnam and P_tmpdir.  */
 #ifndef L_tmpnam
 # include "pathmax.h"
 # define L_tmpnam PATH_MAX
 #endif
+#ifndef P_tmpdir
+# define P_tmpdir "/tmp"
+#endif
 
 typedef struct side SIDE;	/* all variables for one side */
 struct side
@@ -550,6 +552,50 @@ copy_word (SIDE *side, FILE *file)
   side->position++;
 }
 
+/*-------------------------------------------------------------------------.
+| Create a template filename suitable for mkstemp given the temporary	   |
+| directory settings of the system.  The method used here closely follows  |
+| the method used in glibc's tmpfile implementation.			   |
+`-------------------------------------------------------------------------*/
+
+static char *
+create_template_filename (char *tmpl, size_t tmpl_len)
+{
+  struct stat stat_buffer;	/* for checking if file is directory */
+  const char* dir;
+  size_t dirlen;
+
+  dir = getenv ("TMPDIR");
+  if (dir && (stat (dir, &stat_buffer) == 0)
+      && ((stat_buffer.st_mode & S_IFMT) == S_IFDIR))
+    /* nothing */ ;
+  else if ((stat (P_tmpdir, &stat_buffer) == 0)
+           && ((stat_buffer.st_mode & S_IFMT) == S_IFDIR))
+    dir = P_tmpdir;
+  else if ((stat ("/tmp", &stat_buffer) == 0)
+           && ((stat_buffer.st_mode & S_IFMT) == S_IFDIR))
+    dir = "/tmp";
+  else
+    {
+      errno = ENOENT;
+      return NULL;
+    }
+
+  dirlen = strlen (dir);
+  while (dirlen > 1 && dir[dirlen - 1] == '/')
+    dirlen--;			/* remove trailing slashes */
+
+  /* check we have room for "${dir}/wdiff.XXXXXX\0" */
+  if (tmpl_len < dirlen + 1 + 12 + 1)
+    {
+      errno = EINVAL;
+      return NULL;
+    }
+
+  sprintf(tmpl, "%.*s/wdiff.XXXXXX", (int) dirlen, dir);
+  return tmpl;
+}
+
 /*-------------------------------------------------------------------------.
 | For a given SIDE, turn original input file in another one, in which each |
 | word is on one line.							   |
@@ -559,6 +605,7 @@ static void
 split_file_into_words (SIDE *side)
 {
   struct stat stat_buffer;	/* for checking if file is directory */
+  int fd;                /* for file descriptors returned by mkstemp */
 
   /* Open files.  */
 
@@ -570,8 +617,12 @@ split_file_into_words (SIDE *side)
 	 this temporary local file.  Once done, prepare it for reading.
 	 We do not need the file name itself anymore.  */
 
-      tmpnam (side->temp_name);
-      side->file = fopen (side->temp_name, "w+");
+      if (create_template_filename (side->temp_name, L_tmpnam) == NULL)
+	error (EXIT_FAILURE, 0, _("No suitable temporary directory exists"));
+      if ((fd = mkstemp (side->temp_name)) == -1)
+        error (EXIT_FAILURE, errno, "%s", side->temp_name);
+
+      side->file = fdopen (fd, "w+");
       if (side->file == NULL)
 	error (EXIT_FAILURE, errno, "%s", side->temp_name);
       if (unlink (side->temp_name) != 0)
@@ -597,8 +648,12 @@ split_file_into_words (SIDE *side)
   side->character = getc (side->file);
   side->position = 0;
 
-  tmpnam (side->temp_name);
-  side->temp_file = fopen (side->temp_name, "w");
+  if (create_template_filename (side->temp_name, L_tmpnam) == NULL)
+    error (EXIT_FAILURE, 0, _("No suitable temporary directory exists"));
+  if ((fd = mkstemp (side->temp_name)) == -1)
+    error (EXIT_FAILURE, errno, "%s", side->temp_name);
+
+  side->temp_file = fdopen (fd, "w");
   if (side->temp_file == NULL)
     error (EXIT_FAILURE, errno, "%s", side->temp_name);
 
@@ -1128,7 +1183,6 @@ Mandatory arguments to long options are mandatory for short options too.\n\
 \n\
   -C, --copyright            print copyright then exit\n\
   -K, --no-init-term         like -t, but no termcap init/term strings\n\
-  -V, --version              print program version then exit\n\
   -1, --no-deleted           inhibit output of deleted words\n\
   -2, --no-inserted          inhibit output of inserted words\n\
   -3, --no-common            inhibit output of common words\n\
@@ -1140,6 +1194,7 @@ Mandatory arguments to long options are mandatory for short options too.\n\
   -p, --printer              overstrike as for printers\n\
   -s, --statistics           say how many words deleted, inserted etc.\n\
   -t, --terminal             use termcap as for terminal displays\n\
+  -v, --version              print program version then exit\n\
   -w, --start-delete=STRING  string to mark beginning of delete region\n\
   -x, --end-delete=STRING    string to mark end of delete region\n\
   -y, --start-insert=STRING  string to mark beginning of insert region\n\
@@ -1156,7 +1211,7 @@ Report bugs to <wdiff-bugs@gnu.org>.\n"),
 | Main program.	 |
 `---------------*/
 
-void
+int
 main (int argc, char *const argv[])
 {
   int option_char;		/* option character */
@@ -1165,6 +1220,8 @@ main (int argc, char *const argv[])
 
   program_name = argv[0];
   setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
 
   inhibit_left = 0;
   inhibit_right = 0;
@@ -1195,7 +1252,7 @@ main (int argc, char *const argv[])
   count_changed_left = 0;
   count_changed_right = 0;
 
-  while (option_char = getopt_long (argc, argv, "123CKVahilnpstw:x:y:z:",
+  while (option_char = getopt_long (argc, argv, "123CKahilnpstvw:x:y:z:",
 				    longopts, NULL),
 	 option_char != EOF)
     switch (option_char)
@@ -1216,22 +1273,6 @@ main (int argc, char *const argv[])
 	print_copyright ();
 	exit (EXIT_SUCCESS);
 
-      case 'V':
-	printf ("wdiff (GNU %s) %s\n", PACKAGE, VERSION);
-	fputs (_("\
-\n\
-Copyright (C) 1992, 1997 Free Software Foundation, Inc.\n"),
-	     stdout);
-	fputs (_("\
-This is free software; see the source for copying conditions.  There is NO\n\
-warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"),
-	     stdout);
-	fputs (_("\
-\n\
-Written by Franc,ois Pinard <pinard@iro.umontreal.ca>.\n"),
-	     stdout);
-	exit (EXIT_SUCCESS);
-
       case 'a':
 	autopager = 1;
 	break;
@@ -1276,6 +1317,22 @@ Written by Franc,ois Pinard <pinard@iro.umontreal.ca>.\n"),
 	       _("Cannot use -t, termcap not available."));
 #endif
 
+      case 'v':
+	printf ("wdiff (GNU %s) %s\n", PACKAGE, VERSION);
+	fputs (_("\
+\n\
+Copyright (C) 1992, 1997 Free Software Foundation, Inc.\n"),
+	     stdout);
+	fputs (_("\
+This is free software; see the source for copying conditions.  There is NO\n\
+warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"),
+	     stdout);
+	fputs (_("\
+\n\
+Written by Franc,ois Pinard <pinard@iro.umontreal.ca>.\n"),
+	     stdout);
+	exit (EXIT_SUCCESS);
+
       case 'w':
 	user_delete_start = optarg;
 	break;
